/**
 * OW2 Util
 * Copyright (C) 2006-2012 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:EjbJarClassMetadata.java 2372 2008-02-08 18:18:37Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ejbjar.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagementType;

import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.metadata.common.api.interfaces.IEnvEntryAccess;
import org.ow2.util.ee.metadata.common.api.struct.IEnvEntry;
import org.ow2.util.ee.metadata.common.api.struct.IJInterceptors;
import org.ow2.util.ee.metadata.common.api.xml.struct.ISecurityRoleRef;
import org.ow2.util.ee.metadata.ejbjar.api.ClassType;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarClassMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarDeployableMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarFieldMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarMethodMetadata;
import org.ow2.util.ee.metadata.ejbjar.api.IJClassInterceptor;
import org.ow2.util.ee.metadata.ejbjar.api.InterceptorType;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IApplicationException;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IConcurrencyManagementType;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IDependsOn;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJCommonBean;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJEjbAccessTimeout;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJEjbStatefulTimeout;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJLocal;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJMessageDriven;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJRemote;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJSingleton;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJStateful;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJStateless;
import org.ow2.util.ee.metadata.ejbjar.api.struct.ILockType;
import org.ow2.util.ee.metadata.ws.impl.WebservicesCommonClassMetadata;
import org.ow2.util.scan.api.metadata.structures.JClass;

import static javax.ejb.TransactionAttributeType.REQUIRED;
import static javax.ejb.TransactionManagementType.CONTAINER;
import static org.ow2.util.ee.metadata.common.impl.AbsSharedMetadata.concatStringBuilder;

/**
 * This class represents the annotation metadata of a Bean.<br>
 * From this class, we can access to all methods of a bean with its associated
 * information.
 * @param <E> an implementation of EJBDeployable
 * @param <D> an implementation of IEjbJarDeployableMetadata
 * @param <C> an implementation of IEjbJarClassMetadata
 * @param <M> an implementation of IEjbJarMethodMetadata
 * @param <F> an implementation of IEjbJarFieldMetadata
 * @author Florent Benoit
 */
public class EjbJarClassMetadata<
    E extends EJBDeployable<E>,
    D extends IEjbJarDeployableMetadata<E, D, C, M, F>,
    C extends IEjbJarClassMetadata<E, D, C, M, F>,
    M extends IEjbJarMethodMetadata<E, D, C, M, F>,
    F extends IEjbJarFieldMetadata<E, D, C, M, F>>
        extends WebservicesCommonClassMetadata<C, M, F> implements
            IEjbJarClassMetadata<E, D, C, M, F>, IEnvEntryAccess {

    /**
     * Serial version UID.
     */
    private static final long serialVersionUID = -2529693463301260629L;

    /**
     * Parent meta data.
     */
    private transient D ejbJarArchiveMetadata = null;

    /**
     * List of local interfaces.
     */
    private IJLocal jLocal = null;

    /**
     * List of remote interfaces.
     */
    private IJRemote jRemote = null;

    /**
     * CommonBean description.
     */
    private IJCommonBean jCommonBean = null;

    /**
     * Message Driven attribute.
     */
    private IJMessageDriven jMessageDriven = null;

    /**
     * Stateless attribute.
     */
    private IJStateless jStateless = null;

    /**
     * Stateful attribute.
     */
    private IJStateful jStateful = null;


    /**
     * Singleton attribute.
     */
    private IJSingleton jSingleton = null;

    /**
     * Ordered interceptors. If this flag is true it means that
     * interceptor-binding has been used in DD with an interceptor-order
     * element.<br/> Also, this flag is used to check that the ordering is done
     * only once.
     */
    private boolean orderedInterceptors = false;

    /**
     * Local Home.
     */
    private String localHome = null;

    /**
     * Remote Home.
     */
    private String remoteHome = null;

    /**
     * List of annotation interceptors.
     */
    private IJInterceptors annotationInterceptors = null;

    /**
     * List of Singleton dependencies (dependsOn).
     */
    private IDependsOn dependsOn = null;
    
    /**
     * Bean is part of a session synchronization.
     */
    private boolean hasSessionSynchronization = false;
    
    /**
     * EasyBeans global interceptors.<br>
     * These interceptors correspond to a list of interceptors that need to be
     * present first on all methods.
     */
    private List<? extends IJClassInterceptor> globalEasyBeansInterceptors = null;

    /**
     * User interceptors. These interceptors correspond to a list of Interceptor
     * that user has specified in its bean class. It is the interceptors defined
     * in interceptor classes, not the bean class itself. Map&lt;interceptor
     * type &lt;--&gt; List of methods/class corresponding to the
     * interceptor&gt;
     */
    private Map<InterceptorType, List<? extends IJClassInterceptor>> externalUserInterceptors = null;

    /**
     * User interceptors. These interceptors correspond to a list of Interceptor
     * that user has specified in the bean class. It is not defined in separated
     * classes. Map&lt;interceptor type &lt;--&gt; List of methods/class
     * corresponding to the interceptor&gt;
     */
    private Map<InterceptorType, List<? extends IJClassInterceptor>> internalUserInterceptors = null;

    /**
     * Env-entry list.
     */
    private Collection<? extends IEnvEntry> envEntryList;

    /**
     * Transaction management type (default = container).
     */
    private TransactionManagementType transactionManagementType = CONTAINER;

    /**
     * Transaction attribute type (default = required).
     */
    private TransactionAttributeType transactionAttributeType = REQUIRED;

    /**
     * Application exception annotation.
     */
    private IApplicationException applicationException = null;

    /**
     * The type of the class.
     * @see ClassType
     */
    private ClassType classType = null;

    /**
     * Is that the class represented by this metadata has already been modified
     * ?
     */
    private boolean modified = false;

    /**
     * List of roles that are declared on this class.
     */
    private List<String> declareRoles = null;

    /**
     * List of security-role-ref for this class.
     */
    private List<ISecurityRoleRef> securityRoleRefList = null;

    /**
     * List of roles that are allowed on this class/bean.
     */
    private List<String> rolesAllowed = null;

    /**
     * This class/bean has the permitAll annotation.
     */
    private boolean permitAll = false;

    /**
     * &#64;{@link javax.interceptor.ExcludeDefaultInterceptors} method ?
     */
    private boolean excludeDefaultInterceptors = false;

    /**
     * Name of the remote interface (used in EJB 2.1 DD).
     */
    private String remoteInterface = null;

    /**
     * This metadata is linked to a bean.
     */
    private String linkedBeanName = null;

    /**
     * Lock type (default = Write).
     */
    private ILockType lockType = ILockType.WRITE;

    /**
     * ConcurrencyManagement Type (default = container).
     */
    private IConcurrencyManagementType concurrencyManagementType = IConcurrencyManagementType.CONTAINER;

    /**
     * Access Timeout.
     */
    private IJEjbAccessTimeout jEjbAccessTimeout = null;


    /**
     * Stateful Timeout.
     */
    private IJEjbStatefulTimeout jEjbStatefulTimeout = null;

    /**
     * This Singleton needs to be initialized at Startup.
     */
    private boolean isStartupSingleton = false;

    /**
     * True if the class is a local bean.
     */
    private boolean isLocalBean;
    
    /**
     * No-arg Constructor.
     */
    public EjbJarClassMetadata() {
        super();
        this.envEntryList = new ArrayList<IEnvEntry>();
    }

    /**
     * @return the name of the bean on which we are linked.
     * This is used for cloned classmetadata that are for the classes.
     */
    public String getLinkedBean() {
        return linkedBeanName;
    }

    /**
     * This class is linked to the given bean.
     * @param linkedBeanName the bean name
     */
    public void setLinkedBean(final String linkedBeanName) {
        this.linkedBeanName = linkedBeanName;
    }

    /**
     * @return name of the bean (associated to this metadata).
     */
    public String getClassName() {
        return getJClass().getName();
    }

    /**
     * Sets the local interfaces of this class.
     * @param jLocal list of interfaces.
     */
    public void setLocalInterfaces(final IJLocal jLocal) {
        this.jLocal = jLocal;
    }

    /**
     * Sets the remote interfaces of this class.
     * @param jRemote list of interfaces.
     */
    public void setRemoteInterfaces(final IJRemote jRemote) {
        this.jRemote = jRemote;
    }

    /**
     * @return the local interfaces of this class.
     */
    public IJLocal getLocalInterfaces() {
        return this.jLocal;
    }

    /**
     * @return the remote interfaces of this class.
     */
    public IJRemote getRemoteInterfaces() {
        return this.jRemote;
    }

    /**
     * @return true if the class is a stateless class
     */
    public boolean isStateless() {
        return (this.classType != null && this.classType == ClassType.STATELESS);
    }

    /**
     * @return true if the class is a singleton class
     */
    public boolean isSingleton() {
        return (this.classType != null && this.classType == ClassType.SINGLETON);
    }

    /**
     * @return true if the class is a stateful class
     */
    public boolean isStateful() {
        return (this.classType != null && this.classType == ClassType.STATEFUL);
    }

    /**
     * @return true if the class is a session bean class
     */
    public boolean isSession() {
        return (this.classType != null
                && (this.classType == ClassType.STATELESS
                        || this.classType == ClassType.STATEFUL
                        || this.classType == ClassType.SINGLETON));
    }

    /**
     * @return true if the class is an MDB class
     */
    public boolean isMdb() {
        return (this.classType != null && this.classType == ClassType.MDB);
    }

    /**
     * Sets the type of this class.
     * @param cType a type from enum class ClassType.
     * @see org.ow2.easybeans.deployment.annotations.ClassType
     */
    public void setClassType(final ClassType cType) {
        this.classType = cType;
    }

    /**
     * @return Message driven attribute.
     */
    public IJMessageDriven getJMessageDriven() {
        return this.jMessageDriven;
    }

    /**
     * Sets the message driven bean object.
     * @param messageDriven attributes of message driven bean.
     */
    public void setJMessageDriven(final IJMessageDriven messageDriven) {
        this.jMessageDriven = messageDriven;
    }

    /**
     * @return string representation.
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        // classname
        sb.append(this.getClass().getName().substring(this.getClass().getPackage().getName().length() + 1));
        sb.append("[\n");

        // Add super class toString()
        sb.append(super.toString());

        // Class name
        concatStringBuilder("className", getClassName(), sb);

        // superclass name
        concatStringBuilder("superName", getJClass().getSuperName(), sb);

        // interfaces
        concatStringBuilder("interfaces", getJClass().getInterfaces(), sb);

        // classType
        concatStringBuilder("classType", this.classType, sb);

        // jLocal
        concatStringBuilder("jLocal", this.jLocal, sb);

        // jRemote
        concatStringBuilder("jRemote", this.jRemote, sb);

        // jMessageDriven
        concatStringBuilder("jMessageDriven", this.jMessageDriven, sb);

        // remoteHome
        concatStringBuilder("remoteHome", this.remoteHome, sb);

        // localHome
        concatStringBuilder("localHome", this.localHome, sb);

        // transactionManagementType
        concatStringBuilder("transactionManagementType", this.transactionManagementType, sb);

        // transactionAttributeType
        concatStringBuilder("transactionAttributeType", this.transactionAttributeType, sb);

        // annotation Interceptors
        concatStringBuilder("annotationInterceptors", this.annotationInterceptors, sb);

        // lockType
        concatStringBuilder("lockType", this.lockType, sb);

        // concurrencyManagementType
        concatStringBuilder("concurrencyManagementType", this.concurrencyManagementType, sb);

        // depends On
        concatStringBuilder("dependsOn", this.dependsOn, sb);

        // jEjbEJBs
        concatStringBuilder("jAnnotationEJBs", getJEjbEJBs(), sb);

        // jAnnotationResources
        concatStringBuilder("jAnnotationResources", getJAnnotationResources(), sb);

        // javaxPersistencePersistenceContexts
        concatStringBuilder("javaxPersistencePersistenceContexts", getJavaxPersistencePersistenceContexts(), sb);

        // javaxPersistencePersistenceUnits
        concatStringBuilder("javaxPersistencePersistenceUnits", getJavaxPersistencePersistenceUnits(), sb);

        // Methods
        for (M methodAnnotationMetadata : getMethodMetadataCollection()) {
            concatStringBuilder("methods", methodAnnotationMetadata, sb);
        }

        // Fields
        for (F fieldAnnotationMetadata : getStandardFieldMetadataCollection()) {
            concatStringBuilder("fields", fieldAnnotationMetadata, sb);
        }

        // modified
        concatStringBuilder("modified", Boolean.valueOf(this.modified), sb);

        // excludeDefaultInterceptors
        concatStringBuilder("excludeDefaultInterceptors", Boolean.valueOf(this.excludeDefaultInterceptors), sb);

        // declareRoles
        concatStringBuilder("declareRoles", this.declareRoles, sb);

        // rolesAllowed
        concatStringBuilder("rolesAllowed", this.rolesAllowed, sb);

        // permitAll
        concatStringBuilder("permitAll", Boolean.valueOf(this.permitAll), sb);

        // runAs
        concatStringBuilder("runAs", getRunAs(), sb);

        // applicationException
        concatStringBuilder("applicationException", this.applicationException, sb);


        sb.append("]");
        return sb.toString();
    }

    /**
     * @return the &#64;{@link javax.ejb.RemoteHome} class name.
     */
    public String getRemoteHome() {
        return this.remoteHome;
    }

    /**
     * Sets the &#64;{@link javax.ejb.RemoteHome} class name.
     * @param remoteHome the class name.
     */
    public void setRemoteHome(final String remoteHome) {
        this.remoteHome = remoteHome;
    }

    /**
     * @return the &#64;{@link javax.ejb.LocalHome} class name.
     */
    public String getLocalHome() {
        return this.localHome;
    }

    /**
     * Sets the &#64;{@link javax.ejb.LocalHome} class name.
     * @param localHome the class name.
     */
    public void setLocalHome(final String localHome) {
        this.localHome = localHome;
    }

    /**
     * @return transaction management type from
     * @see TransactionManagementType.
     */
    public TransactionManagementType getTransactionManagementType() {
        return this.transactionManagementType;
    }

    /**
     * Sets transaction management type.
     * @see javax.ejb.TransactionManagementType
     * @param transactionManagementType value. (BEAN, CONTAINER)
     */
    public void setTransactionManagementType(final TransactionManagementType transactionManagementType) {
        this.transactionManagementType = transactionManagementType;
    }

    /**
     * @return transaction Attribute type.
     * @see javax.ejb.TransactionAttributeType
     */
    public TransactionAttributeType getTransactionAttributeType() {
        return this.transactionAttributeType;
    }

    /**
     * Set Transaction Attribute Type.
     * @see javax.ejb.TransactionAttributeType
     * @param transactionAttributeType the type of transaction.
     */
    public void setTransactionAttributeType(final TransactionAttributeType transactionAttributeType) {
        this.transactionAttributeType = transactionAttributeType;
    }

    /**
     * @return object representing list of &#64;
     *         {@link javax.interceptor.Interceptors}.
     */
    public IJInterceptors getAnnotationInterceptors() {
        return this.annotationInterceptors;
    }

    /**
     * Sets the object representing the &#64;
     * {@link javax.interceptor.Interceptors} annotation.
     * @param annotationInterceptors list of classes
     */
    public void setAnnotationsInterceptors(final IJInterceptors annotationInterceptors) {
        this.annotationInterceptors = annotationInterceptors;
    }

    /**
     * @return object representing list of &#64;
     *         {@link javax.interceptor.DependsOn}.
     */
    public IDependsOn getDependsOn() {
        return this.dependsOn;
    }

    /**
     * Sets the object representing the &#64;
     * {@link javax.ejb.DependsOn} annotation.
     * @param dependsOn list of beans
     */
    public void setDependsOn(final IDependsOn dependsOn) {
        this.dependsOn = dependsOn;
    }


    /**
     * @return the &#64;{@link javax.ejb.ApplicationException} annotation.
     */
    public IApplicationException getApplicationException() {
        return this.applicationException;
    }

    /**
     * Sets the object representing the &#64;
     * {@link javax.ejb.ApplicationException} annotation.
     * @param applicationException object representation
     */
    public void setApplicationException(final IApplicationException applicationException) {
        this.applicationException = applicationException;
    }

    /**
     * @return true if the classs is a Bean
     */
    public boolean isBean() {
        return isSession() || isMdb() || isManagedBean();
    }

    /**
     * @return array of interfaces name.
     */
    public String[] getInterfaces() {
        return getJClass().getInterfaces();
    }

    /**
     * @return the super class name.
     */
    public String getSuperName() {
        return getJClass().getSuperName();
    }

    /**
     * @return Map&lt;interceptor type &lt;--&gt; List of methods/class
     *         corresponding to the interceptor&gt; (interceptor classes) of
     *         user interceptors that enhancer will use.
     */
    public Map<InterceptorType, List<? extends IJClassInterceptor>> getExternalUserEasyBeansInterceptors() {
        return this.externalUserInterceptors;
    }

    /**
     * Sets the list of user interceptors that enhancers will use.<br>
     * These interceptors are defined outside the bean class (interceptor
     * classes).
     * @param externalUserInterceptors list of interceptors that enhancer will
     *        use.
     */
    public void setExternalUserInterceptors(
            final Map<InterceptorType, List<? extends IJClassInterceptor>> externalUserInterceptors) {
        this.externalUserInterceptors = externalUserInterceptors;
    }

    /**
     * @return Map&lt;interceptor type &lt;--&gt; List of methods/class
     *         corresponding to the interceptor&gt; (bean classes) of user
     *         interceptors that enhancer will use.
     */
    public Map<InterceptorType, List<? extends IJClassInterceptor>> getInternalUserEasyBeansInterceptors() {
        return this.internalUserInterceptors;
    }

    /**
     * Sets the list of user interceptors that enhancers will use.<br>
     * These interceptors are defined in bean class.
     * @param internalUserInterceptors list of interceptors that enhancer will
     *        use.
     */
    public void setInternalUserInterceptors(
            final Map<InterceptorType, List<? extends IJClassInterceptor>> internalUserInterceptors) {
        this.internalUserInterceptors = internalUserInterceptors;
    }

    /**
     * @return list of global interceptors that enhancer will use. (ie : ENC)
     */
    public List<? extends IJClassInterceptor> getGlobalEasyBeansInterceptors() {
        return this.globalEasyBeansInterceptors;
    }

    /**
     * Sets the list of global interceptors that enhancers will use.
     * @param globalEasyBeansInterceptors list of interceptors that enhancer
     *        will use.
     */
    public void setGlobalEasyBeansInterceptors(final List<? extends IJClassInterceptor> globalEasyBeansInterceptors) {
        this.globalEasyBeansInterceptors = globalEasyBeansInterceptors;
    }

    /**
     * @return the attributes for a Stateless/Stateful/MDB
     */
    public IJCommonBean getJCommonBean() {
        return this.jCommonBean;
    }

    /**
     * Sets the attributes for a Stateless/Stateful/MDB.
     * @param commonBean the attributes
     */
    public void setJCommonBean(final IJCommonBean commonBean) {
        // ensure that there is a bean name
        String ejbName = commonBean.getName();
        if (ejbName == null || "".equals(ejbName)) {
            // if not set, it has to be the unqualified name of the bean class.
            // (simplified spec : 10.1.1)
            String className = getClassName();
            commonBean.setName(className.substring(className.lastIndexOf("/") + 1));
        }
        this.jCommonBean = commonBean;
    }

    /**
     * @return the attributes for a Stateful
     */
    public IJStateful getJStateful() {
        return this.jStateful;
    }

    /**
     * Sets the attributes for a Stateful.
     * @param jStateful the attributes
     */
    public void setJStateful(final IJStateful jStateful) {
        this.jStateful = jStateful;
    }

    /**
     * @return the attributes for a Singleton
     */
    public IJSingleton getJSingleton() {
        return this.jSingleton;
    }

    /**
     * Sets the attributes for a Singleton.
     * @param jSingleton the attributes
     */
    public void setJSingleton(final IJSingleton jSingleton) {
        this.jSingleton = jSingleton;
    }


    /**
     * @return the attributes for a Stateless
     */
    public IJStateless getJStateless() {
        return this.jStateless;
    }

    /**
     * Sets the attributes for a Stateless.
     * @param jStateless the attributes
     */
    public void setJStateless(final IJStateless jStateless) {
        this.jStateless = jStateless;
    }

    /**
     * @return true if the class has been modified.
     */
    public boolean wasModified() {
        return this.modified;
    }

    /**
     * Defines that this class has been modified.
     */
    public void setModified() {
        this.modified = true;
    }

    /**
     * Sets the list of roles declared on this class.
     * @param declareRoles the list of roles.
     */
    public void setDeclareRoles(final List<String> declareRoles) {
        this.declareRoles = declareRoles;
    }

    /**
     * @return the list of roles declared on this class.
     */
    public List<String> getDeclareRoles() {
        return this.declareRoles;
    }

    /**
     * Set the list of roles allowed on this class/method.
     * @param rolesAllowed the list of roles.
     */
    public void setRolesAllowed(final List<String> rolesAllowed) {
        this.rolesAllowed = rolesAllowed;
    }

    /**
     * @return the list of roles allowed on this class/method.
     */
    public List<String> getRolesAllowed() {
        return this.rolesAllowed;
    }

    /**
     * This class has PermitAll annotation.
     * @param permitAll the boolean value.
     */
    public void setPermitAll(final boolean permitAll) {
        this.permitAll = permitAll;
    }

    /**
     * @return true if PermitAll annotation.
     */
    public boolean hasPermitAll() {
        return this.permitAll;
    }

    /**
     * This class needs eager initialization.
     * @param startup the boolean value.
     */
    public void setStartup(final boolean startup) {
        this.isStartupSingleton = startup;
    }

    /**
     * @return true if this class(singleton) needs eager initialization.
     */
    public boolean isStartup() {
        return this.isStartupSingleton;
    }

    /**
     * @return true if this class won't use default interceptors.
     */
    public boolean isExcludedDefaultInterceptors() {
        return this.excludeDefaultInterceptors;
    }

    /**
     * Flag this class as a class which exclude default interceptors.
     * @param excludeDefaultInterceptors true if this class is a class which
     *        exclude default interceptors.
     */
    public void setExcludeDefaultInterceptors(final boolean excludeDefaultInterceptors) {
        this.excludeDefaultInterceptors = excludeDefaultInterceptors;
    }

    /**
     * @return true if the interceptors have been ordered for this class.
     */
    public boolean isOrderedInterceptors() {
        return this.orderedInterceptors;
    }

    /**
     * Sets flag to true if interceptors are ordered.
     * @param orderedInterceptors boolean with true/false.
     */
    public void setOrderedInterceptors(final boolean orderedInterceptors) {
        this.orderedInterceptors = orderedInterceptors;
    }

    /**
     * Set the value of the remote Interface property.
     * @param remoteInterface the remote Interface property.
     */
    public void setRemoteInterface(final String remoteInterface) {
        this.remoteInterface = remoteInterface;
    }

    /**
     * @return the name of the remote interface.
     */
    public String getRemoteInterface() {
        return this.remoteInterface;
    }

    /**
     * construct a new JClass.
     * @param array interface list
     */
    public void setInterfaces(final String[] array) {
        JClass jClass = getJClass();
        setJClass(new JClass(jClass.getVersion(), jClass.getAccess(), jClass.getName(), jClass.getSignature(), jClass
                .getSuperName(), array));
    }

    /**
     * {@inheritDoc}
     */
    public D getEjbJarDeployableMetadata() {
        return ejbJarArchiveMetadata;
    }


    /**
     * {@inheritDoc}
     */
    public void setEjbJarArchiveMetadata(final D ejbJarArchiveMetadata) {
        this.ejbJarArchiveMetadata = ejbJarArchiveMetadata;
    }

    /**
     * {@inheritDoc}
     */
    public Collection<? extends IEnvEntry> getEnvEntryCollection() {
        return envEntryList;
    }

    /**
     * @param envEntryCollection a collection of env-entry
     */
    public void setEnvEntryCollection(final Collection<? extends IEnvEntry> envEntryCollection) {
        this.envEntryList = envEntryCollection;
    }


    /**
     * Get a classmetadata linked from this one.
     * @param className the name of the class that is wanted
     * @return the class metadata
     */
    public C getLinkedClassMetadata(final String className) {
        // Check if this class is part of a bean or not
        // If it is linked to a bean, a cloned copy will be obtained, else the scanned class metadata will be retrieved
        if (linkedBeanName != null) {
            return getEjbJarDeployableMetadata().getBeanEjbJarClassMetadataForEjbName(linkedBeanName, className);
        }
        return getEjbJarDeployableMetadata().getScannedClassMetadata(className);
    }

    /**
     * Gets the list of &lt;security-role-ref&gt; elements.
     * @return list of &lt;security-role-ref&gt; elements.
     */
    public List<ISecurityRoleRef> getSecurityRoleRefList() {
        return securityRoleRefList;
    }

    /**
     * Sets the list of &lt;security-role-ref&gt; element.
     * @param securityRoleRefList the list of &lt;security-role-ref&gt; element
     */
    public void setSecurityRoleRefList(final List<ISecurityRoleRef> securityRoleRefList) {
        this.securityRoleRefList = securityRoleRefList;
    }

    /**
     * @return lock type
     * @see javax.ejb.LockType
     */
    public ILockType getLockType() {
        return lockType;
    }

    /**
     * Set Lock Type.
     * @see javax.ejb.LockType
     * @param lockType the type of the lock.
     */
    public void setLockType(final ILockType lockType) {
        this.lockType = lockType;
    }

    /**
     * @return transaction management type from
     * @see javax.ejb.ConcurrencyManagementType.
     */
    public IConcurrencyManagementType getConcurrencyManagementType() {
        return concurrencyManagementType;
    }

    /**
     * Sets concurrency management type.
     * @see javax.ejb.ConcurrencyManagementType
     * @param concurrencyManagementType value.
     *      BEAN,CONCURRENCY_NOT_ALLOWED, CONTAINER
     */
    public void setConcurrencyManagementType(final IConcurrencyManagementType concurrencyManagementType) {
        this.concurrencyManagementType = concurrencyManagementType;
    }

    /**
     * @return true if the class is a local bean.
     */
    public boolean isLocalBean() {
        return isLocalBean;
    }
    
    /**
     * Defines if the bean is a local bean or not.
     * @param isLocalBean true if the bean is a local bean (and expose a no-interface local view)
     */
    public void setLocalBean(boolean isLocalBean) {
        this.isLocalBean = isLocalBean;
    }
    

    /**
     * @return the access timeout on this class/method.
     */
    public IJEjbAccessTimeout getJavaxEjbAccessTimeout() {
        return jEjbAccessTimeout;
    }

    /**
     * Sets the access timeout on this class/method.
     * @param jEjbAccessTimeout information on access timeout.
     */
    public void setJavaxEjbAccessTimeout(IJEjbAccessTimeout jEjbAccessTimeout) {
        this.jEjbAccessTimeout = jEjbAccessTimeout;
    }
    
    /**
     * This bean is linked to a session synchronization and then should report TX lifecycle to the callbacks.
     */
    public void setSessionSynchronization() {
        this.hasSessionSynchronization = true;
    }
    
    /**
     * @return true if the bean is part of a session synchronization
     */
    public boolean hasSessionSynchronization() {
        return hasSessionSynchronization;
    }

    public IJEjbStatefulTimeout getJavaxEjbStatefulTimeout() {
        return jEjbStatefulTimeout;
    }

    public void setJavaxEjbStatefulTimeout(IJEjbStatefulTimeout jEjbStatefulTimeout) {
        this.jEjbStatefulTimeout = jEjbStatefulTimeout;
    }
}
