/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JavaxEjbAccessTimeoutVisitor.java 6236 2012-04-16 10:40:05Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor;

import java.util.concurrent.TimeUnit;

import org.ow2.util.ee.metadata.common.impl.configurator.visitor.AbsAnnotationVisitor;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.AnnotationType;
import org.ow2.util.ee.metadata.ejbjar.api.interfaces.IEjbAccessTimeout;
import org.ow2.util.ee.metadata.ejbjar.api.struct.IJEjbAccessTimeout;
import org.ow2.util.ee.metadata.ejbjar.impl.struct.JEJBAccessTimeout;
import org.ow2.util.scan.api.IType;

/**
 * This class manages the handling of &#64;{@link javax.ejb.AccessTimeout}
 * annotation.
 * @param <T> An implementation of IEjbAccessTimeout interface.
 * @author Florent Benoit
 */
public class JavaxEjbAccessTimeoutVisitor<T extends IEjbAccessTimeout> extends AbsAnnotationVisitor<T> implements
        AnnotationType {

    /**
     * Type of annotation.
     */
    public static final String TYPE = "Ljavax/ejb/AccessTimeout;";

    /**
     * value attribute of the annotation.
     */
    private static final String VALUE = "value";


    /**
     * unit attribute of the annotation.
     */
    private static final String UNIT = "unit";


    /**
     * Access Timeout information.
     */
    private IJEjbAccessTimeout jEjbAccessTimeout = null;

    /**
     * Constructor.
     * @param annotationMetadata linked to a class or method or field metadata
     */
    public JavaxEjbAccessTimeoutVisitor(final T annotationMetadata) {
        super(annotationMetadata);
        jEjbAccessTimeout = new JEJBAccessTimeout();
    }

    /**
     * Visits a primitive value of the annotation.<br>
     * @param name the value name.
     * @param value the actual value, whose type must be {@link Byte},
     *        {@link Boolean}, {@link Character}, {@link Short},
     *        {@link Integer}, {@link Long}, {@link Float}, {@link Double},
     *        {@link String} or {@link IType}.
     */
    @Override
    public void visit(final String name, final Object value) {
        if (VALUE.equals(name)) {
            jEjbAccessTimeout.setValue((Long) value);
        }
    }

    /**
     * Visits an enumeration value of the annotation.
     * @param name the value name.
     * @param desc the class descriptor of the enumeration class.
     * @param value the actual enumeration value.
     */
    @Override
    public void visitEnum(final String name, final String desc, final String value) {
       if (UNIT.equals(name)) {
           jEjbAccessTimeout.setUnit(TimeUnit.valueOf(value.toUpperCase()));
        }
    }

    /**
     * Visits the end of the annotation.<br>
     * Creates the object and store it.
     */
    @Override
    public void visitEnd() {
        // set flag on field
        getAnnotationMetadata().setJavaxEjbAccessTimeout(jEjbAccessTimeout);
    }

    /**
     * @return type of the annotation (its description)
     */
    public String getType() {
        return TYPE;
    }

}
